@@ -4,7 +4,7 @@
 import multiprocessing, threading, traceback, json
 import gobject, dbus, dbus.service
 import sqlite3, mx.DateTime, re, uuid
-import urlshorter, storage, network, util, config
+import urlshorter, storage, network, util, uploader
 from gettext import lgettext as _
 import signal
 
@@ -14,12 +14,15 @@ from util import exceptions
 from util.const import *
 import subprocess
 
+
 try:
   from gi.repository import Unity, Dbusmenu
 except:
   Unity = None
   Dbusmenu = None
 
+from gi.repository import Gio
+
 # Try to import * from custom, install custom.py to include packaging 
 # customizations like distro API keys, etc
 try:
@@ -28,14 +31,12 @@ except:
   pass
 
 try:
-  import indicate
+  from gi.repository import Indicate as indicate
 except:
   indicate = None
 
 gobject.threads_init()
 
-log.logger.name = "Gwibber Dispatcher"
-
 # Dynamically build a list of available service plugins
 PROTOCOLS = {}
 for p in util.resources.get_plugin_dirs()[0]:
@@ -47,12 +48,13 @@ for p in util.resources.get_plugin_dirs()[0]:
 
 FEATURES = json.loads(GWIBBER_OPERATIONS)
 SERVICES = dict([(k, v.PROTOCOL_INFO) for k, v in PROTOCOLS.items()])
-SETTINGS = config.Preferences()
 
-if SETTINGS["interval"] < 5:
-  #log.logger.info( "Migrating refresh interval from %d to %d", SETTINGS["interval"], 5)
-  SETTINGS["interval"] = 5
+gsettings = Gio.Settings.new("org.gwibber.preferences")
 
+if gsettings.get_int("interval") < 5:
+  gsettings.set_int("interval", 5)
+
+log.logger.name = "Gwibber Dispatcher"
 
 def perform_operation((account, opname, args, transient)):
   try:
@@ -73,8 +75,10 @@ def perform_operation((account, opname, args, transient)):
           if isinstance(m, dict) and m.has_key("mid"):
             m["id"] = uuid.uuid1().hex
             m["operation"] = opname
-            m["stream"] = stream
+            m["stream"] = m.get("stream", stream)
             m["transient"] = transient
+            m["time"] = m.get("time", 0)
+            if not m["text"]: m["text"] = ""
             m["rtl"] = util.isRTL(re.sub(text_cleaner, "", m["text"].decode('utf-8')))
             if m.has_key("type"):
               if m["type"] == "link": m["stream"] = "links"
@@ -106,7 +110,7 @@ def perform_operation((account, opname, args, transient)):
             ))
         except Exception as e:
           if not "logtext" in locals(): logtext = "<UNKNOWN>"
-          log.logger.error("%s Operation failed", logtext)
+          log.logger.error("%s Operation failed: %s", logtext, e)
 
     log.logger.debug("%s Finished operation", logtext)
     return ("Success", new_messages)
@@ -253,6 +257,7 @@ class Dispatcher(dbus.service.Object):
     self.bus = dbus.SessionBus()
     bus_name = dbus.service.BusName("com.Gwibber.Service", bus=self.bus)
     dbus.service.Object.__init__(self, bus_name, self.__dbus_object_path__)
+    self.mainloop = loop
 
     self.db = sqlite3.connect(SQLITE_DB_FILENAME)
 
@@ -270,13 +275,26 @@ class Dispatcher(dbus.service.Object):
     self.indicate = None 
 
     if indicate and util.resources.get_desktop_file():
-      self.indicate = indicate.indicate_server_ref_default()
+      self.indicate = indicate.Server.ref_default ()
       self.indicate.set_type("message.gwibber")
       self.indicate.set_desktop_file(util.resources.get_desktop_file())
       self.indicate.connect("server-display", self.on_indicator_server_activate)
       self.indicate.connect("interest-added", self.on_indicator_interest_added)
       self.indicate.connect("interest-removed", self.on_indicator_interest_removed)
       self.indicate.show()
+
+    if Dbusmenu:
+      post_menu = Dbusmenu.Menuitem.new ()
+      post_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Update Status"))
+      post_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
+      post_menu.connect("item-activated", self.show_poster)
+
+      menu_server = Dbusmenu.Server.new("/messaging/commands")
+      root = Dbusmenu.Menuitem.new ()
+      root.child_append (post_menu)
+      menu_server.set_root(root);
+      self.indicate.set_menu (menu_server)
+
     self.indicator_items = {}
     self.notified_items = []
     self.notified_errors = {}
@@ -288,8 +306,14 @@ class Dispatcher(dbus.service.Object):
       self.unseen_counts[s] = 0
 
     if Unity and Dbusmenu:
-      launcher = Unity.LauncherEntry.get_for_desktop_id ("gwibber.desktop")
+      self.launcher = Unity.LauncherEntry.get_for_desktop_id ("gwibber.desktop")
       ql = Dbusmenu.Menuitem.new ()
+      ql_post_menu = Dbusmenu.Menuitem.new ()
+      ql_post_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Update Status"))
+      ql_post_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
+      ql_post_menu.connect("item-activated", self.show_poster)
+
+      ql.child_append (ql_post_menu)
       refresh_menu = Dbusmenu.Menuitem.new ()
       refresh_menu.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Refresh"))
       refresh_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
@@ -310,10 +334,11 @@ class Dispatcher(dbus.service.Object):
       quit_menu.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
       quit_menu.connect("item-activated", self.shutdown)
       ql.child_append (quit_menu)
-      launcher.set_property("quicklist", ql)
+      self.launcher.set_property("quicklist", ql)
+      self.launcher.set_property("count", 0)
+      self.launcher.set_property("count_visible", False)
 
     self.refresh_count = 0
-    self.mainloop = loop
     self.workerpool = multiprocessing.Pool()
 
     self.refresh_timer_id = None
@@ -333,12 +358,29 @@ class Dispatcher(dbus.service.Object):
     self.accounts_service.connect_to_signal("Deleted", self.on_account_deleted)
     self.accounts_service.connect_to_signal("Created", self.on_account_created)
 
+    self.streams_service = util.getbus("Streams")
+    self.streams_service.connect_to_signal("Created", self.perform_single_op)
+
+    self.searches_service = util.getbus("Searches")
+    self.searches_service.connect_to_signal("Created", self.perform_single_op)
+
+  def show_client(self, stream=None, *args):
+    cmd = []
+    cmd.append("gwibber")
+    if stream:
+      cmd.append("-s")
+      cmd.append(stream)
+    subprocess.Popen(cmd, shell=False)
+
   def show_preferences(self, *args):
     subprocess.Popen("gwibber-preferences", shell=False)
 
   def show_accounts(self, *args):
     subprocess.Popen("gwibber-accounts", shell=False)
 
+  def show_poster(self, *args):
+    subprocess.Popen("gwibber-poster", shell=False)
+
   def shutdown(self, *args):
     subprocess.Popen(["pkill", "gwibber"], shell=False)
     self.Quit()
@@ -386,6 +428,13 @@ class Dispatcher(dbus.service.Object):
     except:
       pass
 
+  def perform_single_op (self, data):
+    data = json.loads(data)
+    try:
+      self.PerformOp('{"id": "%s"}' % data["id"])
+    except:
+      pass
+
   @dbus.service.signal("com.Gwibber.Service")
   def LoadingComplete(self):
     self.refreshRunning = False
@@ -405,7 +454,8 @@ class Dispatcher(dbus.service.Object):
             service.Refresh()
 
     """
-    self.refresh()
+    if not self.refreshRunning:
+      self.refresh()
 
   @dbus.service.method("com.Gwibber.Service", in_signature="s")
   def PerformOp(self, opdata):
@@ -470,6 +520,26 @@ class Dispatcher(dbus.service.Object):
     except:
       log.logger.error("Sending failed:\n%s", traceback.format_exc())
 
+  @dbus.service.method("com.Gwibber.Service", in_signature="ss")
+  def Retweet(self, mid, account):
+    log.logger.debug("Retweeting %s", mid)
+    self.PerformOp(json.dumps({
+      "account": account,
+      "operation": "retweet",
+      "args": {"message": {"mid": mid}},
+      "transient": False,
+    }))
+
+  @dbus.service.method("com.Gwibber.Service", in_signature="ss")
+  def Like(self, mid, account):
+    log.logger.debug("Liking %s", mid)
+    self.PerformOp(json.dumps({
+      "account": account,
+      "operation": "like",
+      "args": {"message": {"mid": mid}},
+      "transient": False,
+    }))
+
   @dbus.service.method("com.Gwibber.Service", out_signature="s")
   def GetServices(self):
     """
@@ -507,6 +577,18 @@ class Dispatcher(dbus.service.Object):
     """
     return VERSION_NUMBER
 
+  @dbus.service.method("com.Gwibber.Service", in_signature="s", out_signature="s")
+  def GetAvatarPath(self, url):
+    """
+    Returns the path to the cached avatar image as a string
+    example:
+            import dbus
+            obj = dbus.SessionBus().get_object("com.Gwibber.Service", "/com/gwibber/Service")
+            service = dbus.Interface(obj, "com.Gwibber.Service")
+            avatar = service.GetAvatar(url)
+    """
+    return util.resources.get_avatar_path(url)
+
   @dbus.service.method("com.Gwibber.Service")
   def Start(self):
     """
@@ -563,7 +645,13 @@ class Dispatcher(dbus.service.Object):
     self.notified_errors["account"]["service"] = None
     
   def send_error_notify(self, error):
-    error = json.loads(error)["error"]
+    if not isinstance(error, dict):
+      log.logger.error("Failed to parse error message: %s", error)
+      return
+    if error.has_key("error"):
+      error = json.loads(error)["error"]
+    else:
+      error = json.loads(error)
 
     if self.notified_errors.has_key(error["account"]["service"]):
       if self.notified_errors[error["account"]["service"]] == error["message"]:
@@ -593,6 +681,9 @@ class Dispatcher(dbus.service.Object):
                 self.messages.Message("update", o2[-1])
               else:
                 self.messages.Message("new", o2[-1])
+                for s in "messages", "replies", "private":
+                  if o2[6] == s and o2[9] != 1:
+                    self.unseen_counts[s] = self.unseen_counts[s] + 1
             items.append(o2)
           else:
             errors.append(o2)
@@ -603,14 +694,7 @@ class Dispatcher(dbus.service.Object):
             ",".join(self.messages.columns),
             ",".join("?" * len(self.messages.columns))), items)
 
-      for s in "messages", "replies", "private":
-        if s == "messages": 
-          to_me = 0 
-        else: to_me = 1
-        count = db.execute("SELECT count(mid) FROM messages WHERE stream = ? AND from_me = 0 and to_me = ? AND ROWID > ?", (s,to_me,oldid)).fetchone()[0] or 0
-        self.unseen_counts[s] = self.unseen_counts[s] + int(count)
-  
-      self.update_indicators(self.unseen_counts)
+      gobject.idle_add (self.update_indicators, self.unseen_counts)
 
       new_items = db.execute("""
         select * from (select * from messages where operation == "receive" and ROWID > %s and to_me = 0 ORDER BY time DESC LIMIT 10) as a union
@@ -620,6 +704,15 @@ class Dispatcher(dbus.service.Object):
       for i in new_items:
           self.new_message(i)
     
+      # FIXME: this forces an avatar cache event for transient messages
+      #new_searches = db.execute("""
+      #  select * from (select * from messages where operation == "search" and ROWID > %s and to_me = 0 ORDER BY time DESC) as a union
+      #  select * from (select * from messages where operation IN ("search","user_messages") and ROWID > %s and to_me != 0 ORDER BY time DESC) as b
+      #  ORDER BY time ASC""" % (oldid, oldid)).fetchall()
+      #
+      #for i in new_searches:
+      #    self.new_search_message(i)
+
     for error in errors:
       self.Error(error[1])
       self.send_error_notify(error[1])
@@ -628,6 +721,7 @@ class Dispatcher(dbus.service.Object):
     log.logger.info("Loading complete: %s - %s", self.refresh_count, [o[0] for o in output])
 
   def update_indicators(self, counts):
+    total_unseen = 0
     if indicate:
       if counts.has_key("private"):
         if not self.private_indicator:
@@ -637,6 +731,7 @@ class Dispatcher(dbus.service.Object):
           self.private_indicator.set_property("stream", "private")
           self.private_indicator.show()
         self.private_indicator.set_property("count", str(counts["private"]))
+        total_unseen +- counts["private"]
         if counts["private"] > 0:
           self.private_indicator.set_property_bool("draw-attention", True)
         if self.private_indicator not in self.indicator_items:
@@ -650,6 +745,7 @@ class Dispatcher(dbus.service.Object):
           self.replies_indicator.set_property("stream", "replies")
           self.replies_indicator.show()
         self.replies_indicator.set_property("count", str(counts["replies"]))
+        total_unseen += counts["replies"]
         if self.replies_indicator not in self.indicator_items:
           self.indicator_items["replies"] = self.replies_indicator
         log.logger.debug("Replies Indicator count updated to %s", counts["replies"])
@@ -661,9 +757,18 @@ class Dispatcher(dbus.service.Object):
           self.messages_indicator.set_property("stream", "messages")
           self.messages_indicator.show()
         self.messages_indicator.set_property("count", str(counts["messages"]))
+        total_unseen += counts["messages"]
         if self.messages_indicator not in self.indicator_items:
           self.indicator_items["messages"] = self.messages_indicator
         log.logger.debug("Messages Indicator count updated to %s", counts["messages"])
+    log.logger.debug ("Total unseen count: %d", total_unseen)
+    if Unity:
+      self.launcher.set_property("count", total_unseen)
+      if total_unseen < 1:
+        self.launcher.set_property("count_visible", False)
+      else:
+        self.launcher.set_property("count_visible", True)
+    return False
 
   def on_indicator_interest_added(self, server, interest):
     self.IndicatorInterestAdded()
@@ -676,14 +781,10 @@ class Dispatcher(dbus.service.Object):
     client_bus = dbus.SessionBus()
     log.logger.debug("Raising gwibber client")
     try:
-      client_obj = client_bus.get_object("com.GwibberClient",
-        "/com/GwibberClient", follow_name_owner_changes = True,
-        introspect = False)
-      gw = dbus.Interface(client_obj, "com.GwibberClient")
-      gw.focus_client(reply_handler=self.handle_focus_reply,
-                      error_handler=self.handle_focus_error)
-    except dbus.DBusException:
-      log.logger.error("Indicator activate failed:\n%s", traceback.format_exc())
+      self.handle_indicator_counts()
+    except:
+      pass
+    self.show_client()
 
   def on_indicator_activate(self, indicator, timestamp=None):
     if not indicate: return
@@ -693,14 +794,7 @@ class Dispatcher(dbus.service.Object):
       self.handle_indicator_counts(stream)
     except:
       pass
-    client_bus = dbus.SessionBus()
-    try:
-      client_obj = client_bus.get_object("com.GwibberClient", "/com/GwibberClient")
-      gw = dbus.Interface(client_obj, "com.GwibberClient")
-      gw.show_stream(stream, reply_handler=self.handle_focus_reply,
-                      error_handler=self.handle_focus_error)
-    except dbus.DBusException:
-      log.logger.error("Indicator activation failed:\n%s", traceback.format_exc())
+    self.show_client(stream=stream)
 
   def handle_focus_reply(self, *args):
     log.logger.debug("Gwibber Client raised")
@@ -708,31 +802,57 @@ class Dispatcher(dbus.service.Object):
   def handle_focus_error(self, *args):
     log.logger.error("Failed to raise client %s", args)
 
-  def handle_indicator_counts(self, stream):
-    if not indicate: return
-    if self.indicator_items.has_key(stream):
-      self.indicator_items[stream].set_property("count", str(0))
-      if stream == "private":
-        self.private_indicator.set_property_bool("draw-attention", False)
+  def handle_indicator_counts(self, stream=None):
+    if indicate:
+      if not stream or stream == "home":
+        for s in self.indicator_items.keys():
+          self.indicator_items[s].set_property("count", str(0))
+          self.unseen_counts[s] = 0
+          if s == "private":
+            self.private_indicator.set_property_bool("draw-attention", False)
+        self.launcher.set_property("count", 0)
+        self.launcher.set_property("count_visible", False)
+        return
+      if self.indicator_items.has_key(stream):
+        self.indicator_items[stream].set_property("count", str(0))
+        if stream == "private":
+          self.private_indicator.set_property_bool("draw-attention", False)
+
     self.unseen_counts[stream] = 0
+    if Unity:
+      total_unseen = 0
+      for s in self.unseen_counts.keys():
+        total_unseen += self.unseen_counts[s]
+      log.logger.debug ("handle_indicator_counts total_unseen is %d", total_unseen)
+      self.launcher.set_property("count", total_unseen)
+      if total_unseen < 1:
+        self.launcher.set_property("count_visible", False)
+      else:
+        self.launcher.set_property("count_visible", True)
+    
+
+  def new_search_message(self, data):
+    message = json.loads(data[-1])
+    gobject.idle_add(self.cache_avatar, message)
 
   def new_message(self, data):
     message = json.loads(data[-1])
     if message["transient"]:
       log.logger.debug("Message %s is transient, not notifying", message["id"])
-      return 
+      return
 
     if util.can_notify and  str(message["mid"]) not in self.notified_items:
       self.notified_items.append(message["mid"])
-      if SETTINGS["notify_mentions_only"] and message["to_me"]: 
+      if gsettings.get_boolean("notify-mentions-only") and message["to_me"]: 
         log.logger.debug("%s is a mention and notify_mentions_only is true", message["mid"])
         gobject.idle_add(self.handle_notify_item, message)
-      elif SETTINGS["show_notifications"] and not SETTINGS["notify_mentions_only"]:
+      elif gsettings.get_boolean("show-notifications") and not gsettings.get_boolean("notify-mentions-only"):
         log.logger.debug("%s - show_notifications is true and notify_mentions_only is false", message["mid"])
         gobject.idle_add(self.handle_notify_item, message)
-
+    gobject.idle_add(self.cache_avatar, message)
+    
   def handle_notify_item(self, message):
-    if SETTINGS["show_fullname"]:
+    if gsettings.get_boolean("show-fullname"):
       sender_name = message["sender"].get("name", message["sender"].get("nick", ""))
     else:
       sender_name = message["sender"].get("nick", message["sender"].get("name", ""))
@@ -758,6 +878,11 @@ class Dispatcher(dbus.service.Object):
 
     return False
 
+  def cache_avatar (self, message):
+    if message["sender"].has_key("image"):
+      util.resources.get_avatar_path(message["sender"]["image"])
+    return False
+
   def loading_failed(self, exception, tb):
     self.LoadingComplete()
     log.logger.error("Loading failed: %s - %s", exception, tb)
@@ -773,8 +898,10 @@ class Dispatcher(dbus.service.Object):
     if self.refresh_timer_id:
       gobject.source_remove(self.refresh_timer_id)
 
+    refresh_interval = gsettings.get_int("interval")
+
     if not self.maintRunning and not self.refreshRunning:
-      log.logger.debug("Refresh interval is set to %s", SETTINGS["interval"])
+      log.logger.debug("Refresh interval is set to %s", refresh_interval)
       operations = []
     
       for o in self.collector.get_operations():
@@ -787,7 +914,8 @@ class Dispatcher(dbus.service.Object):
         self.LoadingStarted()
         self.perform_async_operation(operations)
 
-      self.refresh_timer_id = gobject.timeout_add_seconds(int(60 * SETTINGS["interval"]), self.refresh)
+
+      self.refresh_timer_id = gobject.timeout_add_seconds(int(60 * refresh_interval), self.refresh)
     else:
       self.refresh_timer_id = gobject.timeout_add_seconds(int(30), self.refresh)
 
@@ -807,6 +935,9 @@ class ConnectionMonitor(dbus.service.Object):
     self.sysbus = dbus.SystemBus()
 
     self.has_nm = None
+    NM_DBUS_SERVICE = "org.freedesktop.NetworkManager"
+    NM_DBUS_OBJECT_PATH = "/org/freedesktop/NetworkManager"
+    NM_DBUS_INTERFACE = "org.freedesktop.NetworkManager"
 
     try:
       self.nm = self.sysbus.get_object(NM_DBUS_SERVICE, NM_DBUS_OBJECT_PATH)
@@ -815,20 +946,35 @@ class ConnectionMonitor(dbus.service.Object):
     except:
       pass
 
+    self.NM_STATE_UNKNOWN = 0
+
+
+    try:
+      log.logger.debug("NM Version is %s", str(self.nm.Get(NM_DBUS_INTERFACE, "Version")))
+      if str(self.nm.Get(NM_DBUS_INTERFACE, "Version")) >= "0.8.998":
+        log.logger.debug("NM Version is greater than 0.8.997")
+        self.NM_STATE_ASLEEP = 10
+        self.NM_STATE_DISCONNECTED = 20
+        self.NM_STATE_CONNECTING = 40
+        self.NM_STATE_CONNECTED = 70
+      else:
+        log.logger.debug("NM Version is less than 0.8.998")
+        self.NM_STATE_ASLEEP = 1
+        self.NM_STATE_CONNECTING = 2
+        self.NM_STATE_CONNECTED = 3
+        self.NM_STATE_DISCONNECTED = 4
+    except:
+      self.has_nm = False
+
+
+
   def on_connection_changed(self, state):
     log.logger.debug("Network state changed, new state is %d", state)
-    """
-    NM_STATE_UNKNOWN = 0
-    NM_STATE_ASLEEP = 1
-    NM_STATE_CONNECTING = 2
-    NM_STATE_CONNECTED = 3
-    NM_STATE_DISCONNECTED = 4
-    """
 
-    if state == NM_STATE_CONNECTED:
+    if state == self.NM_STATE_CONNECTED:
       log.logger.info("Network state changed to Online")
       self.ConnectionOnline()
-    else:
+    elif state == self.NM_STATE_DISCONNECTED:
       log.logger.info("Network state changed to Offline")
       self.ConnectionOffline()
 
@@ -844,7 +990,7 @@ class ConnectionMonitor(dbus.service.Object):
       log.logger.info("Can't determine network state, assuming online")
       return True
     try:
-      if self.nm.state() == NM_STATE_CONNECTED:
+      if self.nm.state() == self.NM_STATE_CONNECTED:
         return True
       else:
         return False
@@ -871,7 +1017,7 @@ class URLShorten(dbus.service.Object):
             short_url = shortener.Shorten(url)
     """
     
-    service = SETTINGS["urlshorter"] or "is.gd"
+    service = gsettings.get_string("urlshorter") or "is.gd"
     log.logger.info("Shortening URL %s with %s", url, service)
     if self.IsShort(url): return url
     try:
@@ -885,6 +1031,51 @@ class URLShorten(dbus.service.Object):
         return True
     return False
 
+class Uploader(dbus.service.Object):
+  __dbus_object_path__ = "/com/gwibber/Uploader"
+
+  def __init__(self):
+    self.bus = dbus.SessionBus()
+    bus_name = dbus.service.BusName("com.Gwibber.Uploader", bus=self.bus)
+    dbus.service.Object.__init__(self, bus_name, self.__dbus_object_path__)
+
+  @dbus.service.method("com.Gwibber.Uploader", in_signature="s", out_signature="")
+  def Upload(self, filepath):
+    """
+    Takes a file path as a string and returns a url
+    example:
+            import dbus
+            filepath = "/home/ken/Documents/awesomeimage.jpg"
+            obj = dbus.SessionBus().get_object("com.Gwibber.Uploader", "/com/gwibber/Uploader")
+            uploader = dbus.Interface(obj, "com.Gwibber.Uploader")
+            
+            def done(path, url):
+                if path == filepath:
+                    sig_complete.remove()
+                    sig_failed.remove()
+                    print "Uploaded to", url
+            
+            def failed(path, message):
+                if path == filepath:
+                    sig_complete.remove()
+                    sig_failed.remove()
+                    print "Upload failed with message", message
+            
+            sig_complete = uploader.connect_to_signal("UploadComplete", done)
+            sig_failed = uploader.connect_to_signal("UploadFailed", failed)
+            uploader.Upload(filepath)
+    """
+    log.logger.info("Uploading image %s", filepath)
+    url = uploader.upload(filepath, self.UploadComplete, self.UploadFailed)
+
+  @dbus.service.signal(dbus_interface="com.Gwibber.Uploader", signature="ss")
+  def UploadComplete(self, filepath, public_url):
+    log.logger.info("Image %s uploaded as %s", filepath, public_url)
+
+  @dbus.service.signal(dbus_interface="com.Gwibber.Uploader", signature="ss")
+  def UploadFailed(self, filepath, error_message):
+    log.logger.info("Image %s failed to upload with message '%s'", filepath, error_message)
+
 class Translate(dbus.service.Object):
   __dbus_object_path__ = "/com/gwibber/Translate"
 
