@@ -44,6 +44,37 @@ class MessageManager(dbus.service.Object):
       self.db.execute("CREATE TABLE messages (%s)" % schema) 
       self.db.execute("create unique index idx1 on messages (mid, account, operation, transient)")
 
+  def maintenance(self):
+    log.logger.info("Cleaning up database...")
+    accounts = self.db.execute("SELECT distinct(account) FROM messages").fetchall()
+    for acct in accounts:
+      try:
+        if not self.db.execute("SELECT count(id) FROM accounts WHERE id = '%s'" % acct[0]).fetchone()[0]:
+          log.logger.info("DB Maintenance: Found data for an unknown account %s, removing", acct[0])
+          self.db.execute("DELETE FROM messages WHERE account = '%s'" % acct[0])
+      except:
+        pass
+      try:
+        count = self.db.execute("SELECT count(data) FROM messages WHERE operation = 'receive' AND stream = 'messages' AND account = '%s'" % acct[0]).fetchone()[0]
+        log.logger.info("Found %d records in the messages stream for account %s", count, acct[0])
+        if count > 2000:
+          log.logger.info("Purging old data for %s", acct[0])
+          self.db.execute("DELETE FROM messages WHERE account = ? AND operation = 'receive' AND stream = 'messages' AND time IN (SELECT CAST (time AS int) FROM (SELECT time FROM messages WHERE account = ? AND operation = 'receive' AND stream = 'messages' AND time != 0 ORDER BY time ASC LIMIT (SELECT COUNT(time) FROM messages WHERE operation = 'receive' AND stream = 'messages' AND account = ? AND time != 0) - 2000) ORDER BY time ASC)", (acct[0],acct[0],acct[0]))
+      except:
+        pass
+    self.db.execute("VACUUM")
+    return
+  
+
+  @dbus.service.signal("com.Gwibber.Messages", signature="ss")
+  def Message(self, change, data): pass
+
+  @dbus.service.method("com.Gwibber.Messages", in_signature="s", out_signature="s")
+  def Get(self, id):
+    results = self.db.execute("SELECT data FROM messages WHERE id=?", (id,)).fetchone()
+    if results: return results[0]
+    return ""
+
 class SearchManager(dbus.service.Object):
   __dbus_object_path__ = "/com/gwibber/Searches"
 
@@ -155,22 +186,42 @@ class StreamManager(dbus.service.Object):
     results = self.db.execute("SELECT data FROM streams")
     return "[%s]" % ", ".join([i[0] for i in results.fetchall()])
 
-  @dbus.service.method("com.Gwibber.Streams", in_signature="ssd", out_signature="s")
-  def Messages(self, stream, account, time):
+  @dbus.service.method("com.Gwibber.Streams", in_signature="ssdssssd", out_signature="s")
+  def Messages(self, stream, account, time, transient, recipient, orderby, order, limit):
+    if stream == "home":
+      stream = "all"
+      account = "all"
+      
     with self.db:
-      if (stream != "all") and (account != "all"):
-        query = "SELECT data FROM messages WHERE stream = ? AND account = ? AND time > ? AND transient = 0 LIMIT 50"
-        results = self.db.execute(query, (stream, account, time))
+      if (stream == "sent") and (account != "all"):
+        query = "SELECT data FROM messages WHERE from_me = 1 AND account = ? AND time >= ? AND transient = ? order by %s %s" % (orderby, order)
+        if limit > 0: query = query + " LIMIT %d" % limit
+        results = self.db.execute(query, (account, time, transient))
+      elif (stream == "sent") and (account == "all"):
+        query = "SELECT data FROM messages WHERE from_me = 1 AND time >= ? AND transient = ? order by %s %s" % (orderby, order)
+        if limit > 0: query = query + " LIMIT %d" % limit
+        results = self.db.execute(query, (time, transient))
+      elif (transient != "0") and (account != "all") and recipient != "0":
+        query = "SELECT data FROM messages WHERE time >= ? AND transient = ? OR (time >= ? AND account = ? AND recipient = ?) order by %s %s" % (orderby, order)
+        if limit > 0: query = query + " LIMIT %d" % limit
+        results = self.db.execute(query, (time, transient, time, account, recipient))
+      elif (stream != "all") and (account != "all"):
+        query = "SELECT data FROM messages WHERE stream = ? AND account = ? AND time >= ? AND transient = ? order by %s %s" % (orderby, order)
+        if limit > 0: query = query + " LIMIT %d" % limit
+        results = self.db.execute(query, (stream, account, time, transient))
       elif (stream == "all") and (account != "all"):
-        query = "SELECT data FROM messages WHERE account = ? AND time > ? AND transient = 0 LIMIT 50"
-        results = self.db.execute(query, (account, time))
+        query = "SELECT data FROM messages WHERE account = ? AND time >= ? AND transient = ? order by %s %s" % (orderby, order)
+        if limit > 0: query = query + " LIMIT %d" % limit
+        results = self.db.execute(query, (account, time, transient))
       elif (stream != "all") and (account == "all"):
-        query = "SELECT data FROM messages WHERE stream = ? AND time > ? AND transient = 0 LIMIT 50"
-        results = self.db.execute(query, (stream, time))
+        query = "SELECT data FROM messages WHERE stream = ? AND time >= ? AND transient = ? order by %s %s" % (orderby, order)
+        if limit > 0: query = query + " LIMIT %d" % limit
+        results = self.db.execute(query, (stream, time, transient))
       else:
-        query = "SELECT data FROM messages WHERE time > ? AND transient = 0 LIMIT 50"
-        results = self.db.execute(query, (time,))
-    return json.dumps("[%s]" % ", ".join([i[0] for i in results.fetchall()]))
+        query = "SELECT data FROM messages WHERE time >= ? AND transient = ? order by %s %s" % (orderby, order)
+        if limit > 0: query = query + " LIMIT %d" % limit
+        results = self.db.execute(query, (time, transient))
+    return "[%s]" % ", ".join([i[0] for i in results.fetchall()])
 
   @dbus.service.method("com.Gwibber.Streams", in_signature="s", out_signature="s")
   def Create(self, search):
