@@ -61,7 +61,11 @@ import mimetypes
 import pycurl
 import StringIO
 
-from const import proxy_factory
+try:
+  #import libproxy
+  libproxy = None
+except:
+  libproxy = None
 
 # try to use simplejson first, otherwise fallback to XML
 RESPONSE_FORMAT = 'JSON'
@@ -1115,14 +1119,15 @@ class Facebook(object):
         c.setopt(pycurl.HTTPHEADER, ["Accept:"])
         c.setopt(c.HTTPPOST, [(x,str(y)) for x,y in post_data.items()])
 
-        if proxy_factory:
+        if libproxy:
+            proxy_factory = libproxy.ProxyFactory()
             proxylist = proxy_factory.getProxies(FACEBOOK_SECURE_URL if secure else FACEBOOK_URL)
             if proxylist:
                 proxy = proxylist[0]
                 if (proxy.find("@") != -1):
-                    self.curl.setopt(pycurl.PROXYAUTH, ["CURLAUTH_ANY"])
+                    c.setopt(pycurl.PROXYAUTH, ["CURLAUTH_ANY"])
                 if (proxy.find("direct://") != 0):
-                    self.curl.setopt(pycurl.PROXY, proxy)
+                    c.setopt(pycurl.PROXY, proxy)
 
         b = StringIO.StringIO()
         c.setopt(pycurl.WRITEFUNCTION, b.write)
