@@ -1,4 +1,7 @@
-import gtk, dbus, resources, os, mx.DateTime, webbrowser
+import gtk, dbus, os, mx.DateTime, webbrowser
+import subprocess
+import gobject
+from microblog.util import resources
 from microblog.util.const import *
 # Try to import * from custom, install custom.py to include packaging 
 # customizations like distro API keys, etc
@@ -7,6 +10,7 @@ try:
 except:
   pass
 
+from microblog.util import resources
 import gettext
 from gettext import ngettext
 
@@ -51,6 +55,32 @@ def get_theme_colors(w = gtk.Window()):
 def pixbuf(path):
   return gtk.gdk.pixbuf_new_from_file(resources.get_ui_asset(path))
 
+def icon(name, size=16, use_theme=True):
+  if use_theme:
+    theme_path = resources.get_theme_path(resources.THEME_NAME)
+    if theme_path:
+      fname = os.path.join(theme_path, name)
+      for ext in [".svg", ".png"]:
+        if os.path.exists(fname + ext):
+          return fname + ext
+
+  theme = gtk.icon_theme_get_default()
+  finfo = theme.lookup_icon(name, size, 0)
+  return finfo.get_filename() if finfo else None
+
+def get_service_icon(name, size=16):
+  theme = gtk.icon_theme_get_default()
+  finfo = theme.lookup_icon(name, size, 0)
+  if finfo:
+    return finfo.get_filename()
+  else:
+    finfo = resources.get_ui_asset("icons/breakdance/%sx%s/%s.png" % (size,size,name))
+    if finfo:
+      return finfo
+    else:
+      finfo = resources.get_ui_asset("icons/%sx%s/%s.png" % (size,size,name))
+      return finfo
+
 def nop(*a): pass
 
 load_url = webbrowser.open
@@ -90,9 +120,36 @@ def generate_time_string(t):
   elif d.seconds < 3600 and d.seconds >= 60:
     minutes = round(d.seconds / 60)
     return gettext.ngettext("%(minute)d minute ago", "%(minute)d minutes ago", minutes) % {"minute": minutes}
-  elif round(d.seconds) < 60:
+  elif d.seconds < 60:
     seconds = round(d.seconds)
     if seconds < 0: return gettext.gettext("Just now")
     return gettext.ngettext("%(sec)d second ago", "%(sec)d seconds ago", seconds) % {"sec": seconds}
   else: return "BUG: %s" % str(d)
 
+def show_accounts_dialog(account=None, message=None, obj=None, callback=None):
+  cmd = []
+  if account:
+    cmd.append("-a")
+    cmd.append(account)
+  if message:
+    cmd.append("-c")
+    cmd.append("error")
+    cmd.append("-m")
+    cmd.append(message)
+  if os.path.exists(os.path.join("bin", "gwibber-accounts")):
+    cmd.insert(0, os.path.join("bin", "gwibber-accounts"))
+  else:
+    cmd.insert(0, "gwibber-accounts")
+  print cmd
+  process = subprocess.Popen(cmd)
+  gobject.timeout_add(500, check_ret, process, obj, callback) 
+
+def check_ret(process, obj=None, callback=None):
+  if callback == None:
+    return False
+  if process.poll() == 0:
+    callback()
+    return False
+  else:
+    return True
+
